/* RIDDLEBOX MAIN FILE

   Includes setup and task flow.
   All other menues and tasks/riddles stored in functions

   For more information please visit: https://www.instructables.com/id/Arduino-Riddlebox

   Licence: CC-BY-SA
   https://creativecommons.org/licenses/by-sa/2.5/
   //th 01/2018
*/
//Including required libraries
#include <Wire.h>
#include <LiquidCrystal_I2C.h>
#include <EEPROM.h>
#include <Keypad.h>
#include <Servo.h>

//Defining keypad layout
const byte ROWS = 4; // Four rows
const byte COLS = 4; // Three columns
//Define the Keymap here:
char keys[ROWS][COLS] = {
  {'1', '2', '3', 'A'},
  {'4', '5', '6', 'B'},
  {'7', '8', '9', 'C'},
  {'*', '0', '#', 'D'}
};
// Connect keypad ROW0, ROW1, ROW2 and ROW3 to these Arduino pins.
byte rowPins[ROWS] = { 8, 7, 6, 5 };
// Connect keypad COL0, COL1 and COL2 to these Arduino pins.
byte colPins[COLS] = { 12, 11, 10, 9 };

//Creates keypad element
Keypad kpd = Keypad( makeKeymap(keys), rowPins, colPins, ROWS, COLS );

//Creates lcd element and sets I2C address
LiquidCrystal_I2C lcd(0x3F, 20, 4);

//Creates servo elements
Servo doorServo;
Servo lockServo;

//Creates and sets variables
  //Control lines for servos
byte doorServoPin = 45; //Choose any PWM enabled pin on your Arduino
byte lockServoPin = 44; //Choose and PWM enabled pin on your Arduino
  //Pins for the keyswitch
byte keySwitch1 = 22;   //Any pin on the Arduino should do
byte keySwitch2 = 23;   //Any pin on the Arduino should do
byte switchState1;
byte switchState2;
byte oldSwitchState1;
byte oldSwitchState2;
int eeAddr = 0;
  //Progess initialisation. If line EEPROM.update in line 147 is commented out, this can be used for debugging
byte progress = 0;
char key;

void setup() {
  //pinMode init
  pinMode(keySwitch1, INPUT_PULLUP);
  pinMode(keySwitch2, INPUT_PULLUP);

  //Servo init. Door closes and locks
  doorServo.attach(doorServoPin);
  doorServo.write(90);
  delay(500);
  lockServo.attach(lockServoPin);
  lockServo.write(10);

  //Serial init for debugging
  //Serial.begin(9600);

  //EEPROM read (Use EEPROM.write to perform reset headlessly)
  //EEPROM.write(eeAddr, progress);
  progress = EEPROM.read(eeAddr);

  //LCD init and splash screen
  lcd.backlight();
  lcd.init();
  lcd.noCursor();
  lcd.clear();
  lcd.setCursor(7, 1);
  lcd.print("Andi's");
  lcd.setCursor(5, 2);
  lcd.print("Riddlebox");
  //If box has been used (i.e. tutorial completed) Welcome back is added to splash sequence
  if (progress != 0) {
    delay(2000);
    lcd.clear();
    lcd.setCursor(4, 1);
    lcd.print("Welcome back");
  }

  delay(2000);
  //Holding deined button during startup opens settings menu
  char key = kpd.getKey();
  if (key) {
    if (key = 'A') { //Defines button for settings menu
      advanced(); //Calls advanced menu for settings
    }
  }
}

//Calls current riddle or menu as function, then updates progress in EEPROM
void loop() {
  switch (progress) {
    case 0:
      intro();
      break;
    case 1:
      elementary();
      break;
    case 2:
      outsourcing();
      break;
    case 3:
      sixteen();
      break;
    case 4:
      choices();
      break;
    case 5:
      offline();
      break;
    case 6:
      invisible();
      break;
    case 7:
      calculus();
      break;
    case 8:
      change();
      break;
    case 9:
      patience();
      break;
    case 10:
      theAnswer();
      break;
    case 11:
      endScreen();
      break;
  }
  //Updates progress in EEPROM
  EEPROM.update(eeAddr, progress);
  //Debugging serial output
  //Serial.println(progress);
  delay(250);
}
